<?php

namespace Packages\Product\Http\Controllers;

use Illuminate\Support\Facades\Auth;
use App\Http\Controllers\Controller;
use Facade\Ignition\Support\Packagist\Package;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Support\Facades\Facade;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;
use Packages\Product\Models\Draft;
use Packages\Product\Http\Requests\ProductRequest;
use Packages\Product\Http\Requests\ProductRequestEdit;
use Packages\Product\Models\Product;
use Illuminate\Http\Request;
use App\Models\Category;
use App\Models\Upload;
use phpDocumentor\Reflection\Types\Compound;

class ProductController extends Controller
{
    public function __construct()
    {
    }

//  *******************************************show  page product ************
    public function index()
    {
        $categories = Category::orderBy('id', 'DESC')->get();
        $draft = Draft::orderBy('id', 'DESC')->first();
        $products = Product::query()->orderBy('id', 'DESC');

        if (request()->has('productCategory') && request('productCategory') != '')
            $products = $products->whereHas('categories', function (Builder $query) {
                $query->whereRaw('id = ?', [request('productCategory')]);
            });
        if (request()->has('productStatus') && request('productStatus') != '')
            $products = $products->whereRaw('status = ?', [request('productStatus')]);
        if (request()->has('productName') && request('productName') != '')
            $products = $products->whereRaw('title = ?', [request('productName')]);
        if (request()->has('creatorId') && request('creatorId') != '')
            $products = $products->whereHas('user', function (Builder $query) {
                $query->whereRaw('name= ?', [request('creatorId')]);
            });
        $products = $products->paginate();
        return view('product::index', compact('products', 'categories', 'draft'));
    }

//**************************************************** page create products *****************
    public function create(Request $request)
    {
        return view('product::product.create');
    }
// *********************************************************save products ******************
    public function store(ProductRequest $request)
    {
        $data = [
            'title' => $request->title,
            'slug' => $request->slug,
            'ReleaseStatus' => $request->ReleaseStatus,
            'type' => $request->type,
            'discription' => $request->discription,
            'creator_id' => auth()->user()->id,
        ];
        $product = Product::create($data);
        $view = 'create';
        if ($product->type === 'ساده'){
            $view = 'saveProductStepTwo';
        }elseif ($product->type ==='متغیر'){
            $view = 'productStepVariable';
        }elseif ($product->type === 'باندل'){
            $view = 'productStepVariable';
        }
        return redirect()->route($view, $product->id);
    }
    public function productStepTwo($id)
    {
        $product = $this->productChecker($id);
        if ($product != false){
            $msg = 'ذخیره محصول با موفقیت انجام شد ';
            return view('product::product.simpleProduct', compact('product'))->with('success', $msg);
        }else{
            return abort(419);
        }
    }

    public function productStepVariable($id)
    {
        $product = $this->productChecker($id);
        if ($product != false){
            $msg = 'ذخیره محصول با موفقیت انجام شد ';
            return view('product::product.variableProduct', compact('product'))->with('success', $msg);
        }else{
            return abort(419);
        }
    }

    public function productStepThree(Request $request)
    {
//        dd($request->all());
        $product = $this->productChecker($request->product_id);
        if ($product != false){
            $request->validate([
                'sku' => 'required',
                'status' => 'required',
                'price' => 'required',
                'sale_price' => 'required',
            ]);
            /**
             * update product
             */
            $data = [
                'sku' => $request->sku,
                'status' => $request->status,
                'price' => $request->price,
                'sale_price' => $request->sale_price,
            ];
            $product->update($data);
            $msg = 'ذخیره محصول با موفقیت انجام شد ';
            return redirect(route('products.index'))->with('success', $msg);
        }else{
            return abort(419);
        }
    }

    public function productChecker($id)
    {
        $product = Product::where('id', $id)->where('creator_id', Auth::user()->id)->first();
        if (!empty($product)){
            return $product;
        }else{
            return false;
        }
    }
    // ******************************************************edit page product*********************************************
    public function edit(Product $product)
    {
        return view('product::product.edit', compact('product', 'categories'));
    }

// *******************************************update page product*****************************************
    public function update(ProductRequestEdit $request, Product $product)
    {
        $request->merge([
            'price' => preg_replace('/[^0-9]+/', '', $request->price),
            'sale_price' => preg_replace('/[^0-9]+/', '', $request->sale_price),
        ]);
        $data = [
            'title' => $request->title,
            'slug' => $request->slug,
            'sku' => $request->sku,
            'price' => $request->price,
            'sale_price' => $request->sale_price,
            'type' => $request->type,
            'ReleaseStatus' => $request->ReleaseStatus,
            'status' => $request->status,
            'discription' => $request->discription,
            'creator_id' => auth()->user()->id,
        ];


        if ($request->has('gallery_image')) {
            $file = $request->only('gallery_image', 'captionGallery', 'descriptionGallery');
            $type = 'gallery_image';
            $diskName = 'product';
            $this->updateGallery($file, $diskName, $product, $type);
        }
        if ($request->has('featured_image')) {
            $info = $request->only(['featureDescription', 'featureCaption']);
            $file = $request->file('featured_image');
            $type = 'featured_image';
            $diskName = 'product';
            $this->updateFeature($file, $diskName, $product, $type, $info);
        }
        $product->update($data);
        $product->categories()->sync($request->categories);
        $msg = 'ویرایش محصول با موفقیت انجام شد ';
        return redirect(route('products.index'))->with('success', $msg);
    }

    //***************************** delete products*****************************************
    public function destroy(Product $product)
    {
        $product->delete();
        {
            return back();
        }
    }
//****************upload images in the product************************
    public function uploadFeature(Request $request)
    {
        if ($request->has('featured_image')) {
            $info = $request->only(['featureDescription', 'featureCaption']);
            $file = $request->file('featured_image');
            $type = 'featured_image';
            $diskName = 'product';
            $fileExtension = $file->getClientOriginalExtension();
            $fileMimeType = $file->getMimeType();
            $afterDiskRoot = '/' . jdate()->format('Y') . '/' . jdate()->format('m');
            $fileName = jdate(time())->format('Ymd') . '_' . $file->getClientOriginalName();
            $filePath = storage_path('app/public/' . $diskName . $afterDiskRoot . '/' . $fileName);

            if (File::exists($filePath)) {
                $fileName = time() . '_' . $fileName;
            }
            $draft = Draft::orderBy('id', 'Desc')->get()->first();
            $upload = $file->storeAs($afterDiskRoot, $fileName, $diskName);

            $uploadData = [
                'name' => $fileName,
                'path' => $upload,
                'mime_type' => $fileMimeType,
                'extension' => $fileExtension,
                'parent_type' => 'packages\product',
                'uploadable_type' => 'Packages\Product\Models\Product',
                'uploadable_id' => $draft->id,
                'type' => $type,
                'descriptionImg' => $info['featureDescription'],
                'caption' => $info['featureCaption'],
            ];
            $uploads = Upload::where('type', $type)->where('uploadable_id', $draft->id)->first();
            if (empty($uploads)) {
                $uploaded = Upload::create($uploadData);
            } else {
                $uploaded = $uploads->update($uploadData);
            }
        }
    }


    public function updateFeature($file, $diskName, $product, $type, $info)
    {
        $fileExtension = $file->getClientOriginalExtension();
        $fileMimeType = $file->getMimeType();
        $afterDiskRoot = '/' . jdate()->format('Y') . '/' . jdate()->format('m');
        $fileName = jdate(time())->format('Ymd') . '_' . $file->getClientOriginalName();
        $filePath = storage_path('app/public/' . $diskName . $afterDiskRoot . '/' . $fileName);

        if (File::exists($filePath)) {
            $fileName = time() . '_' . $fileName;
        }

        $upload = $file->storeAs($afterDiskRoot, $fileName, $diskName);
        $uploadData = [
            'name' => $fileName,
            'path' => $upload,
            'mime_type' => $fileMimeType,
            'extension' => $fileExtension,
            'type' => $type,
            'descriptionImg' => $info['featureDescription'],
            'caption' => $info['featureCaption'],
        ];
        $uploaded = $product->uploads()->update($uploadData);
    }

//************************delete image in the product**********************
    public function destroyFile(Upload $upload)
    {
        $upload->delete();
        $msg = 'حذف عکس محصول با موفقیت انجام شد ';
        session()->flash('image_deleted', $msg);
        return redirect()->back();
    }

//    ****************************uploadGallery********************
    public function uploadGallery(Request $request)
    {

        if ($request->has('gallery_image')) {
            $files = $request->only('gallery_image', 'captionGallery', 'descriptionGallery');
            $type = 'gallery_image';
            $diskName = 'product';
//           dd($files);
      foreach ($files['gallery_image'] as $key => $file) {
            $fileExtension = $file->getClientOriginalExtension();
            $fileMimeType = $file->getMimeType();
            $afterDiskRoot = '/' . jdate()->format('Y') . '/' . jdate()->format('m');
            $fileName = jdate(time())->format('Ymd') . '_' . $file->getClientOriginalName();
            $filePath = storage_path('app/public/' . $diskName . $afterDiskRoot . '/' . $fileName);

            if (File::exists($filePath)) {
                $fileName = time() . '_' . $fileName;
            }
          $draft = Draft::orderBy('id', 'Desc')->get()->first();
            $upload = $file->storeAs($afterDiskRoot, $fileName, $diskName);

            $uploadData = [
                'name' => $fileName,
                'path' => $upload,
                'mime_type' => $fileMimeType,
                'extension' => $fileExtension,
                'parent_type' => 'packages\product',
                'uploadable_type' => 'Packages\Product\Models\Product',
                'uploadable_id' => $draft->id,
                'type' => $type,
                'descriptionImg' => $files['descriptionGallery'][$key],
                'caption' => $files['captionGallery'][$key],
            ];
//         dd($uploadData);
//          $uploaded = Upload::where('uploadable_id', $draft->id)->update($uploadData);
           $uploaded = Upload::create($uploadData);
          dd($uploaded);

        }
      }
    }

    public function updateGallery($files, $diskName, $product, $type)
    {
        foreach ($files['gallery_image'] as $key => $file) {
            $fileExtension = $file->getClientOriginalExtension();
            $fileMimeType = $file->getMimeType();
            $afterDiskRoot = '/' . jdate()->format('Y') . '/' . jdate()->format('m');
            $fileName = jdate(time())->format('Ymd') . '_' . $file->getClientOriginalName();
            $filePath = storage_path('app/public/' . $diskName . $afterDiskRoot . '/' . $fileName);
            if (File::exists($filePath)) {
                $fileName = time() . '_' . $fileName;
            }
            $upload = $file->storeAs($afterDiskRoot, $fileName, $diskName);
            $uploadData = [
                'name' => $fileName,
                'path' => $upload,
                'mime_type' => $fileMimeType,
                'extension' => $fileExtension,
                'type' => $type,
                'descriptionImg' => $files['descriptionGallery'][$key],
                'caption' => $files['captionGallery'][$key],
            ];
            $uploaded = $product->uploads()->update($uploadData);
        }
    }
}
